// Bezeichnung: Applet
// Autor      : Peter Weigel

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import pefas.sort.*;

public class vesaapplet extends Applet implements MouseListener, Runnable {
  protected ptfield field;  // unser Feld
  protected int state = -1; // Zustnde: -1 start, -2 on initfield, -3 on sort, 0 sorted, 1 randomfill, 2 sort_down, 3 unsorted
  protected int fieldsize;  // merkt sich Feldgre
  protected isort sortalg;  // merkt sich Sortieralgorithmus
  protected Thread th = null; // Sortier-Thread

  public vesaapplet() {
    // Initialisierung
    super();
    setBackground(SystemColor.control);
    setLayout (new BorderLayout());
    field = new ptfield(8192);
    BorderPanel sortpanel = new BorderPanel(new BorderLayout());
    sortpanel.setInsets(3);
    sortpanel.add(field, "Center");
    add(sortpanel, "Center");
    field.addMouseListener(this);
  }

  public void init() { // Applet initialisieren
    // Parameter holen, evtl. Standardvorgabe verwenden
    String algorithm = getParameter("algorithm");
    if (algorithm == null) { algorithm = "bubblesort"; }
    String delay = getParameter("delay");
    if (delay == null) { delay = "0"; }
    String size = getParameter("size");
    if (size == null) { size = "512"; }

    // letzte Vorbereitung (statische Einstellungen)
    try {
      field.setDelay(Integer.parseInt(delay)*1000);
      sortalg = sortbucket.newInstance(algorithm);
      fieldsize = Integer.parseInt(size);
    } catch (Exception e) { sortalg = null; }
    if (sortalg == null) {
      field.setBackground(SystemColor.control); // da gibts nix zu tun, das zeigen wir auch
    }
  }

  public void initfield(int typ) {
    state = -2; // initfield: wir sind gerade dabei
    if (typ == 2) {
      field.sortedfill_dn(fieldsize); // Absteigend sortieren
    } else {
      field.randomfill(fieldsize);    // vllig unsortiert (random)
      typ = 1;
    }
    state = typ; // Zustand merken (1,2)
  }

  public void sortfield() {
    state = -3; // Sortieren: die Show beginnt
    (th = new Thread(this)).start();
  }

  public void run() {
    try {
      field.setBackground(new Color(240,240,200));  // Hintergrund etwas ndern
      sortalg.sort(field);                          // Sortieren starten
      state = 0;                                    // und bei Erfolg Zustand ndern
    } catch (RuntimeException e) { state = 3; };     // beim Abbruch natrlich auch
    field.setBackground(new Color(255,255,200));    // Hintergrund wieder normal
    th = null;
  }

  public void start() { // Applet starten
    if ((sortalg != null) && (state >= -1)) {
      initfield(1); // erste Fllung: randomfill
    }
  }

  public void stop() { // Applet stoppen
    if (state == -3) {
      field.abort();   // Sortierung abbrechen
      try { th.join(); } catch (InterruptedException e) {};
    }
    System.gc();
  }

  public void mouseClicked(MouseEvent e) {
    if (state >= 0) { // Wartezustand?
      if ((state == 0) || ((e.getModifiers() & e.BUTTON3_MASK) != 0)) {
        initfield(state+1); // wenn sortiert oder rechte Maustaste
      } else if ((state >= 1) && ((e.getModifiers() & e.BUTTON1_MASK) != 0)) {
        sortfield(); // wenn linke Maustaste und unsortiert
      }
    } else if ((state == -3) && ((e.getModifiers() & e.BUTTON1_MASK) != 0)) {
      field.abort(); // wenn linke Maustaste und beim Sortieren
    }
  }

  public void mouseExited(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}

  public String getAppletInfo() {
    return "Title   : "+title()+"\nVersion: "+version()+"\nAutor   : "+autor()+"\n";
  }

  public String[][] getParameterInfo() {
    String[][] info = {{"algorithm", "bubblesort, shakersort, ...", "Name des Sortierverfahrens"},
                       {"delay","Mikrosekunden","Zeitverzgerung pro Vergleich/Vertauschung"},
                       {"size","1 bis 8192","Feldgre"}};
    return info;
  }

  public static String title()   {return "VESA - Visualisierung einzelner Sortieralgorithmen";}
  public static String version() {return "1.03 (05.03.2002)";}
  public static String autor()   {return "Peter Weigel";}
}
